/**
 * File:
 *  KnightRider.c
 *
 * Notes:
 *  Simple demo created for the easyPic7 board.
 *
 * Ports B, C & D:
 *      SW3: 2, 3 and 4 ON
 *
 * Turn on LCD backlight:
 *      SW4: 6 On
 *
 * History:
 *  29/04/2012 Written by Simon Platten
 *  02/06/2012 Moved animation to just ports C and D
 *  06/07/2012 Added variable speed control via POT
 */
#include <built_in.h>

#include "anim.h"
#include "7SegLED.h"
#include "lcd.h"
// Constants
#define TIMER_FREQUENCY         ((uint)5000) // Hz
#define DELAY_BETWEEN_LEDS      (TIMER_FREQUENCY / 25)
#define MAX_SAMPLES             5
#define TOTAL_LED_SEQC          (sizeof(aryLEDseqC) / sizeof(aryLEDseqC[0]))
#define TOTAL_LED_SEQD          (sizeof(aryLEDseqD) / sizeof(aryLEDseqD[0]))

static volatile uint uintPostDelay = DELAY_BETWEEN_LEDS;

static volatile outputScheduleDef aryLEDseqC[] = {
  { 0 }, { 1 }, { 2 }, { 3 }, { 4 }, { 5 }, { 6 }, { 7 } };
static volatile outputScheduleDef aryLEDseqD[] = {
  { 0 }, { 1 }, { 2 }, { 3 }, { 4 }, { 5 }, { 6 }, { 7 } };
static volatile animDef animC =
  { &LATC, TOTAL_LED_SEQC, 3, aryLEDseqC };
static volatile animDef animD =
  { &LATD, TOTAL_LED_SEQD, 3, aryLEDseqD };
/**
 * Interrupt handler
 */
void interrupt() {
  playAnimation(&animC);

  TMR0H = 0xFF;    // preset for Timer0 MSB register
  TMR0L = 0xCE;    // preset for Timer0 LSB register
  TMR0IF_bit = 0;
}
/**
 * Entry point
 */
void main() {
  int intSteps;
// Set-up hardware
#ifdef P18F45K22
  ANSELA = 0x02;                 // Configure RA1 pin as analog
  ANSELB = 0;                    // Configure PORTB pins as digital
  ANSELC = 0;                    // Configure PORTC pins as digital
  ANSELD = 0;                    // Configure PORTD pins as digital
#endif
#ifdef P18F4550
  ADCON1 = 0x0f;                 // Configure all ports as digital
#endif
  TRISA = 0x02;                  // Set RA1 pin as input
  TRISB = 0;                     // Configure PORTB as output
  TRISC = 0;                     // Configure PORTC as output
  TRISD = 0;                     // Configure PORTD as output
  LATB = 0;                      // Clear PORTC
  LATC = 0;                      // Clear PORTC
  LATD = 0;                      // Clear PORTD
// Initialise the LCD display
  initLCD();
// Display the application title
  lcdText(LCD_ROW_1 | LCD_CENTER, "Knight Rider");
  lcdText(LCD_ROW_2 | LCD_CENTER, "v2 07/07/2012");
// Set-up delay pointers
  for( intSteps=0; intSteps<TOTAL_LED_SEQC; intSteps++ ) {
    aryLEDseqC[intSteps].puintPostDelay = &uintPostDelay;
  }
  for( intSteps=0; intSteps<TOTAL_LED_SEQD; intSteps++ ) {
    aryLEDseqD[intSteps].puintPostDelay = &uintPostDelay;
  }
// Initialise counter
//  ulngCtr = 0;
// Create fader schedule
  intSteps = createSchedule(5, 0);
// Create and initilise the faders
  initAnim(&animC, intSteps);
  initAnim(&animD, intSteps);
// Join animation sequences to form one sequence
  joinAnims(&animC, &animD);
// Timer0 Registers:
// 16-Bit Mode; Prescaler=1:32; 
// TMRH Preset=FF; TMRL Preset=CE; Freq=5,000.00Hz; Period=0.20 ms
  T0CON.TMR0ON = 1;// Timer0 On/Off Control bit:1=Enables Timer0 / 0=Stops Timer0
  T0CON.T08BIT = 0;// Timer0 8-bit/16-bit Control bit: 1=8-bit timer/counter / 0=16-bit timer/counter
  T0CON.T0CS   = 0;// TMR0 Clock Source Select bit: 0=Internal Clock (CLKO) / 1=Transition on T0CKI pin
  T0CON.T0SE   = 0;// TMR0 Source Edge Select bit: 0=low/high / 1=high/low
  T0CON.PSA    = 0;// Prescaler Assignment bit: 0=Prescaler is assigned; 1=NOT assigned/bypassed
  T0CON.T0PS2  = 1;// bits 2-0  PS2:PS0: Prescaler Select bits
  T0CON.T0PS1  = 0;
  T0CON.T0PS0  = 0;
  TMR0H = 0xFF;    // preset for Timer0 MSB register
  TMR0L = 0xCE;    // preset for Timer0 LSB register
  TMR0IE_bit = 1;
// Global interrupt enable
  GIE_bit = 1;
// Endless loop
  while (1) {
// Set the delay according to the ADC setting
    uintPostDelay = min(900, max(1, ADC_Read(1)));
  }
}